@@ -19,12 +19,13 @@ Step 2: Run the test:
 	  test_pyenvlib --mylib --verify # disassemble verify 'mylib'
 """
 
-import py_compile, re, tempfile
 import os.path as osp
+import py_compile
+import re
+import tempfile
 
 import xdis.magics as magics
 from xdis.load import check_object_path
-from xdis import IS_PYPY
 
 # ----- configure this for your needs
 
@@ -49,13 +50,16 @@ PYPY_TEST_VERSIONS = (
 # -----
 
 
-import os, time, shutil, sys
+import os
+import shutil
+import sys
+import time
 from fnmatch import fnmatch
 
-from xdis import disassemble_file, PYTHON3, PYTHON_VERSION
+from xdis import PYTHON3, PYTHON_VERSION_TRIPLE, disassemble_file
 from xdis.verify import verify_file
 
-LONG_PYTHON_VERSION = "%s.%s.%s" % (
+LONG_PYTHON_VERSION = (
     sys.version_info[0],
     sys.version_info[1],
     sys.version_info[2],
@@ -175,6 +179,8 @@ def do_tests(
     for i, bc_file in enumerate(files):
         if verbose:
             print(os.path.join(src_dir, bc_file))
+        if src_dir.find("simple_source") >= 0:
+            continue
         if sys.version_info >= (3, 4, 0) and bc_file.endswith(".py"):
             check_bc_file = check_object_path(bc_file)
             if not osp.exists(check_bc_file):
@@ -209,7 +215,7 @@ if __name__ == "__main__":
     test_version = None
 
     if len(sys.argv) == 1:
-        sys.argv[1] == "--simple"
+        sys.argv.append("--simple")
 
     test_options_keys = list(test_options.keys())
     test_options_keys.sort()
@@ -234,7 +240,7 @@ if __name__ == "__main__":
         pass
 
     if test_version == "simple":
-        if PYTHON_VERSION > 2.6:
+        if PYTHON_VERSION_TRIPLE > (2, 6):
             test_dirs.append((os.path.join(my_dir, "simple_2.7"), PY, "simple-source"))
             pass
         pass
